"""Top-level package for Light Controller."""

__author__ = """Tristan Cotte"""
__email__ = 'tristan.cotte@sgs.com'
__version__ = '0.1.0'

import time
from typing import List

import serial
from serial import SerialException

from .transformations import declist_to_binlist, bin_to_hex
from .utils_uart import split, compute_checksum, compute_command


class LightManager(serial.Serial):
    def __init__(self, port: str, baudrate: int, list_channels: List[int]):
        """
        Class which enables to handle the 8 lights channels thanks to the "FG-PDV400W-24-8T" controller
        :param port: communication port used by the computer to reach the controller. Ex : "COM1"
        :param baudrate: communication frequency used to communicate with the controller
        :param list_channels: list of channels used by the controller to handle lights
        """
        super().__init__(port, baudrate, stopbits=serial.STOPBITS_ONE, parity=serial.PARITY_NONE)
        self.port = port
        self.baudrate = baudrate
        self.list_channels = list_channels

        self.initialization()

    def initialization(self) -> None:
        """
        Steady mode on all the channels -> command Hx.1 for all channels named x
        8 is the parameter of the mode
        """
        for i in self.list_channels:
            chain = "$8" + str(i) + "000"
            chain_list = split(chain)
            list_bin = declist_to_binlist(chain_list)

            checksum = compute_checksum(list_bin)
            checksum_hex = bin_to_hex(checksum)
            command = chain + checksum_hex
            self.write(command.encode())

    def switch_off(self) -> None:
        """
        Switch off all lights in the platform
        """
        for c in self.list_channels:
            command = compute_command(intensity=0, channel=c)
            self.write(command.encode())

    def switch_on(self, intensities, channels=None) -> None:
        """
        Switch on one or several lights sending several commands of intensity for the lights
        :param channels: channels of the lights which change value
        :param intensities: list of the light's intensity with index respective to the channel list
        """
        if channels is None:
            channels = self.list_channels

        if isinstance(channels, list):
            for c, i in zip(channels, intensities):
                command = compute_command(intensity=i, channel=c)
                self.write(command.encode())

        elif isinstance(channels, int):
            command = compute_command(intensity=intensities, channel=channels)
            self.write(command.encode())




from typing import List

import numpy as np
from numpy import ndarray

from .transformations import dec_to_hex, declist_to_binlist, bin_to_hex


def split(word: str) -> List[str]:
    """
    Split a word into list of character : "word" -> ["w", "o", "r", "d"]
    :param word: input string
    :return : list with one letter of input string by element
    """
    return [char for char in word]


def string_to_upperlist(x: str) -> List[str]:
    """
    Transform a string in upper list : "word" -> ["W", "O", "R", "D"]
    :param x: input string
    :return : list with one upper letter of input string by element
    """
    list_intensity = split(x)
    list_intensity = [x.upper() for x in list_intensity]
    return list_intensity


def compute_checksum(list_bin: List[str]) -> str:
    """
    Compute the checksum from the list of binary numbers. It consists in make a XOR on all the binary numbers in the
    list
    :param list_bin: list of binary numbers
    :return : checksum string
    """
    list_checksum = compute_xor_in_col_array(list_bin)
    list_checksum = [str(x) for x in list_checksum]
    checksum = "".join(list_checksum)
    return checksum


def compute_xor_in_col_array(list_bin: List[str]) -> ndarray:
    """
    Make a XOR on all digits of a column in an array of binary numbers
    :param list_bin: list of string with 8 binary characters
    :return : ndarray of 8 binary integers
    """
    list_checksum = np.zeros(8, dtype=int)
    for x in range(len(list_bin[0])):
        for y in range(len(list_bin)):
            list_checksum[x] = list_checksum[x] ^ int(list_bin[y][x])
    return list_checksum


def compute_command(intensity: int, channel: int) -> str:
    """
    Compute the command for an intensity and a channel entered
    :param intensity: intensity of lighting command [0; 255]
    :param channel: channel at which the user applied the command
    :return: UART command
    """
    if not isinstance(channel, str):
        channel = str(channel)

    entry_char = "$"
    brightness_char = "3"

    intensity_hex = dec_to_hex(intensity)
    list_intensity = string_to_upperlist(intensity_hex)

    chain = entry_char + brightness_char + channel + "0" + "".join(list_intensity)

    chain_list = [entry_char, brightness_char, channel, 0] + list_intensity

    list_bin = declist_to_binlist(chain_list)
    checksum = compute_checksum(list_bin)
    checksum_hex = bin_to_hex(checksum)

    command = chain + checksum_hex
    return command



from typing import List


def adjust_len_bin(x: str) -> str:
    """
    Get 0 at the beggining of a chain to get 8 caracters : "1000" -> "00001000"
    :param x: string integer
    :return: string of 8 characters filled out with "0" on the left
    """
    while len(str(x)) != 8:
        x = "0" + str(x)
    return x


def dec_to_bin(x: int) -> int:
    """
    Transform decimal number to binary number
    :param x: decimal number
    :return: binary number
    """
    return int(bin(x)[2:])


def bin_to_hex(x: str) -> str:
    """
    :param x: binary string
    :return: hex number upper characters
    """
    x = hex(int(x, 2))[-2:]
    return x.upper()


def dec_to_hex(x: int) -> str:
    """
    :param x: decimal number
    :return: hexadecimal number in string format
    """
    x_hex = hex(int(x))
    # if the binary value of x is included in [0-8]
    if len(x_hex) != 4:
        x_hex = x_hex[:2] + "0" + x_hex[-1:]
    return x_hex[-2:]


def bin_to_ascii(x: str) -> int:
    """
    :param x: binary number in integer format
    :return: ascii number in int format
    """
    return ord(str(x))


def declist_to_binlist(chain_list: List[str]) -> List[str]:
    """
    This function changes the list of binary values in ascii values. Then, it transforms the ascii decimal values in
    binary strings
    :param chain_list: list of binary values
    :return: list of binary values
    """
    list_bin = []

    for value in chain_list:
        value_ascii = bin_to_ascii(value)
        ascii_bin = str(dec_to_bin(value_ascii))

        if len(ascii_bin) < 8: ascii_bin = adjust_len_bin(ascii_bin)
        list_bin.append(ascii_bin)
    return list_bin
